package gears.book.chap3;

/**
 * データの保存先となるオブジェクトです。<br/>
 * このサンプルアプリケーションでは、実行が容易なように、データベースサーバを用いません。
 * よって、アプリケーションサーバを再起動するとデータは初期化されます。<br/>
 * このクラスはシングルトンです。
 * @author Shumpei Shiraishi
 */
public class DataStore {
	private static final DataStore instance = new DataStore();
	
	// 保存するデータ
	private Data data = new Data();
	
	private DataStore() {
	}
	/**
	 * このクラスの唯一のインスタンスを取得します。
	 * @return このクラスのインスタンス
	 */
	public static DataStore getInstance() {
		return instance;
	}
	
	/**
	 * このクラスが保持しているデータを取得します。
	 * @return データ
	 */
	public synchronized Data loadData() {
		return data;
	}
	
	/**
	 * データを保存します。<br/>
	 * 保存するデータは、既存のデータとバージョン番号の比較が行われ、バージョン番号が
	 * 異なっていた場合例外が発生します。
	 * @param newData 保存する新しいデータ
	 * @throws ConflictException すでに保存されているデータとバージョン番号が異なるデータを
	 *  保存しようとした場合
	 */
	public synchronized void saveData(Data newData) throws ConflictException {
		// バージョンが異なる・・・コンフリクト
		if (data.getVersion() != newData.getVersion()) {
			throw new ConflictException();
		}
		this.data = newData;
		// バージョン番号をインクリメント
		data.setVersion(data.getVersion() + 1);
	}
}
